TEXT_FONT = 'color="#333"'

# stuff like {\tt blah blah}
FONTS = {'tt':'<font size=+2><tt>#1</tt></font>',
         'em':'<i>#1</i>',
         'bf':'<b>#1</b>',
         'dred':'<font color="red">#1</font>'}

# stuff like \QQ
MACROS = {r'\abcd':r'\mathrm{matrix}',
          r'\CC':r'\mathbf{C}',
          r'\QQ':r'\mathbf{Q}',
          r'\ZZ':r'\mathbf{Z}',
          r'\FF':r'\mathbf{F}',
          r'\PP':r'\mathbf{P}',
          r'\TT':r'\mathbf{T}',
          r'\cM':r'\mathcal{M}',
          r'\SL':r'\mathrm{SL}',
          r'\h':r'\mathbf{h}',
          r'\Ker':r'\mathrm{Ker}',          
          r'\End':r'\mathrm{End}',          
          r'\mod':r'\mathrm{mod}',          
          r'\tensor':r'\otimes',          
          r'\ncisom':r'\approx',          
          r'\xrightarrow':r'\rightarrow',          
          r'\nmid':r'\not\mid',
          r'\eps':r'\varepsilon',
          r'\#':'#',
          r'\_':'_',
          r'\maketitle':'',
          r'\tableofcontents':'',
          r'\newpage':'',
          r'\ref':'',
          r'\begin{theorem}':'<br><br><b>Theorem.</b> <i><br><br>',
          r'\end{theorem}':'</i>',
          r'\begin{abstract}':'<center>',
          r'\end{abstract}':'</center>',
          r'\begin{quote}':'<pre>',
          r'\end{quote}':'</pre>',
          r'\begin{exercise}':'<h2>Exercise</h2><i>',
          r'\end{exercise}':'</i>',
          '``':'"',
          "''":'"',
          r'\begin{equation}':'$$',
          r'\end{equation}':'$$',
          }

# stuff like \url{http://sagemath.org} or \section{foo}
MACROS1 = {r'\url':'<a target="_new" href="#1">#1</a>',
           r'\paper':'',
           r'\label':'',
           r'\todo':'',
           r'\footnote':'',
           r'\endnote':'',
           r'\section*':'<font size=+3 color="darkgreen"><h1 align=center>#1</h1></font>',
           r'\section':'<font  size=+3 color="darkgreen"><h1  align=center>#1</h1></font>',
           r'\subsection':'<br><br><br><br><br><br><br><br><br><br><br><br><hr><font color="blue"><h2 align=center>#1</h2></font>',
           r'\slide':'<br><br><br><br><br><br><br><br><br><br><br><br><hr><hr><font color="blue"><h3 align=center>#1</h3><hr><hr></font>',
           r'\eqref':'',
           r'\noindent':'',
           r'\talk':'#1',
           r'\worksheet':''
           }

def worksheet(z, ws):
    i = z.lower().find('\\worksheet{%s}'%ws.lower())
    if i == -1:
        raise RuntimeError, 'No such worksheet: "%s"'%ws
    j = z[i+1:].find('\\worksheet')
    if j == -1:
        j = len(z)
    else:
        j += i+1
    return '<font %s>%s</font>'%(TEXT_FONT, z[i:j])
    

def wholedoc(z):
    begin = '\\begin{document}'
    i = z.find(begin)
    if i == -1:
        return 'Untitled\nsystem:sage\n' + z
    i += len(begin) + 1

    title = 'Untitled'
    j = z.find('\\title{')
    if j != -1:
        k = z[j:].find('}')
        if k != -1:
            title = z[j + len('\\title{'):j + k]
    title = title.replace(r'\\',' ')
    return '%s\nsystem:sage\n<font %s>%s</font>'%(title, TEXT_FONT, z[i:])

def inline_verbatim(z):
    # replace all \verb[symbol]stuff[symbol] inline stuff
    # by <pre>stuff</pre>.
    verb = '\\verb'
    i = 0
    while True:
        w = in_verbatim(z, i)
        if w != -1:
            i = w
            continue
        j = z[i:].find(verb)
        if j == -1:
            return z
        j += i
        symbol = z[j+len(verb)]
        k = z[j+len(verb)+1:].find(symbol)
        if k == -1:
            return z
        k += j + len(verb)+1
        stuff = z[j+len(verb)+1:k]
        stuff = stuff.replace('<','&lt;')
        z = z[:j] + FONTS['tt'].replace('#1', stuff) + z[k+1:]

def in_verbatim(z, i):
    """
    Return position after verbatim if position i
    in the string z is in a verbatim block.  Otherwise
    return -1.
    """
    k = z[:i].rfind('\\begin{verbatim}')
    if k != -1:
        # maybe in verbatim
        m = z[k:].find('\\end{verbatim}')
        if i <= m: # we were in a verbatim environment
            return m + k + 1 # continue from outside it
    return -1

def paragraphs(z):
    """
    Replace all new paragraph empty lines not in verbatim
    environments by the HTML <p>.
    """
    i = 0
    done = False
    v = z.splitlines()
    cnt = 0
    for i in range(len(v)):
        cnt += len(v[i])
        if len(v[i].strip()) == 0 and (
          i == len(v)-1 or len(v[i+1].strip()) != 0):
            # if not in verbatim, then split
            if in_verbatim(z, cnt) == -1:
                v[i] = '<p>'
    return '\n'.join(v)

def remove_comments(z):
    """
    Remove all % latex comments not in verbatim environments.
    """
    i = 0
    while True:
        j = z[i:].find('%')
        if j == -1:
            return z
        j += i
        # delete to end of line unless we
        # happen to be in a verbatim enviro.
        w = in_verbatim(z, i)
        if w != -1:
            i = w
            continue
        # find end of line with %
        m = z[j:].find('\n')
        if m != -1:
            m += j
        else:
            m = len(z)
        z = z[:j] + z[m:]

def input_lines(z):
    """
    Given the inside of a verbatim block, return what it should
    get replaced by.
    """
    if '@interact' in z:
        # special case -- return everything in case of an @interact block.
        return z
    
    def is_input(n):
        return n.lstrip().startswith('..x.') or n.lstrip().startswith('sage:')
    
    v = z.split('\n')
    for i in range(len(v)-1,0,-1):
        if is_input(v[i]):
            v[i] = v[i].lstrip().lstrip('..x.').lstrip('sage:').strip()
        else:
            if i == 0 or is_input(v[i-1]):
                v[i] = '}}}\n{{{'
            else:
                v[i] = ''
    return '\n'.join([x for x in v if x.strip()])

def parse_verbatim_blocks(r):
    r = r.replace(r'\begin{verbatim}','</font>\n{{{')
    r = r.replace(r'\end{verbatim}','}}}\n<font %s>'%TEXT_FONT)
    r = r.replace('%link','')
    i = 0
    while True:
        j = r[i:].find('{{{')
        if j == -1: break
        j += i
        k = r[j:].find('}}}')
        if k == -1: break
        k += j
        r = r[:j] + input_lines(r[j:k]) + '\n' + r[k:]
        i = k+1
    return r

def parse_fonts(z):
    for latex, html in FONTS.iteritems():
        i = 0
        while True:
            srch = '\\%s'%latex
            j = z[i:].find(srch)
            if j == -1:
                break
            j += i
            k = in_verbatim(z, j)
            if k != -1:
                i = k
                continue
            # so, not in verbatim
            k = z[j:].find('}')
            if k == -1:
                break
            k += j
            inside = z[j+len(srch):k]
            z = '%s%s%s'%(z[:j-1], html.replace('#1',inside), z[k+1:])
            i = k+1
    return z

def balanced_find(s, left, right):
    """
    Think of left and right as left and right parens.
    This finds the closing paren, assuming possible open
    and closes in the expression.
    """
    cnt = 0
    for i in range(len(s)):
        if s[i] == left:
            cnt += 1
        elif s[i] == right:
            cnt -= 1
            if cnt == 0:
                return i
    return -1

def parse_macros(z):
    for x,y in MACROS.iteritems():
        z = z.replace(x, y)
        
    for macro, y in MACROS1.iteritems():
        i = 0
        while True:
            j = z[i:].find(macro)
            if j == -1:
                break
            j += i
            w = in_verbatim(z, j)
            if w != -1:
                i = w
                continue
            k = balanced_find(z[j:],'{','}')
            if k == -1:
                break
            k += j
            arg = z[j+len(macro)+1:k]
            z = z[:j] + y.replace('#1',arg) + z[k+1:]
            i = k + 1
    return z

def to_worksheet(f, out, ws=None):
    r = open(f).read()
    if worksheet is None:
        # output worksheet is whole document
        r = wholedoc(r)
    else:
        # output just the given worksheet
        r = ws + '\nsystem:sage\n' + worksheet(r, ws)
    r = remove_comments(r)
    r = inline_verbatim(r)
    r = paragraphs(r)
    r = parse_fonts(r)
    r = parse_macros(r)
    r = parse_verbatim_blocks(r)
    open(out,'w').write(r)

def make():
    for ws, file, n in [
        ('Bordeaux 1: Introduction', 'intro', 292),
        ('Bordeaux 2 -- Number Fields: Creating Number Fields', 'nf', 293),
        ('Bordeaux 3 -- Number Fields: Galois Groups and Class Groups', 'nf', 294),
        ('Bordeaux 4 -- Number Fields: Orders and Relative Extensions', 'nf', 295),
        ('Bordeaux 5 -- A Birds Eye View', 'bird', 296),
        ('Bordeaux 6 -- Modular Forms', 'modforms', 298),
        ]:
        print ws, file, n
        to_worksheet(file + '.tex',
                     '/home/wstein/misc/misc/worksheets/admin/%s/worksheet.txt'%n,
                     ws)    

